#ifndef MSP_CORE_MAPUTILS_H_
#define MSP_CORE_MAPUTILS_H_

#include <stdexcept>
#include <typeinfo>

namespace Msp {

class key_error: public std::runtime_error
{
public:
	key_error(const std::type_info &);
	~key_error() throw() { }
};


template<typename T>
typename T::mapped_type &get_item(T &map, const typename T::key_type &key)
{
	typename T::iterator i = map.find(key);
	if(i==map.end())
		throw key_error(typeid(T));

	return i->second;
}

template<typename T>
const typename T::mapped_type &get_item(const T &map, const typename T::key_type &key)
{
	typename T::const_iterator i = map.find(key);
	if(i==map.end())
		throw key_error(typeid(T));

	return i->second;
}

template<typename T>
const typename T::iterator insert_unique(T &map, const typename T::key_type &key, const typename T::mapped_type &item)
{
	if(map.count(key))
		throw key_error(typeid(T));

	return map.insert(typename T::value_type(key, item)).first;
}

} // namespace Msp

#endif
